%%% -------------------------------------------------------------------
%%% Description :
%%%
%%% Created : 27.04.2012
%%% -------------------------------------------------------------------
-module(e).
%%% Author  : rekby

-behaviour(gen_server).
%% --------------------------------------------------------------------
%% Include files
%% --------------------------------------------------------------------
-define(TABLE_COUNT, 100).
-define(TABLE_PREFIX, "ets_").

%% --------------------------------------------------------------------
%% External exports
-export([get_count/0, put/1, process/2, start_link/0]).

%% gen_server callbacks
-export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).

-record(state, {}).

%% ====================================================================
%% External functions
%% ====================================================================
get_count(0)->
	0;
get_count(TableNumber)->
	StringNumber = integer_to_list(TableNumber),
	TableName = ?TABLE_PREFIX ++ StringNumber,
	TableAtom = list_to_existing_atom(TableName),
	ets:info(TableAtom, size) + get_count(TableNumber - 1).

get_count()->
	get_count(?TABLE_COUNT).

put(Data)->
	TableNumber = random:uniform(?TABLE_COUNT),
	StringNumber = integer_to_list(TableNumber),
	TableName = ?TABLE_PREFIX ++ StringNumber,
	TableAtom = list_to_existing_atom(TableName),
	ets:insert(TableAtom, Data).

process(Key, F) ->
	DataList = ets:lookup(et, Key),
	ets:delete(et, Key),
	F(DataList).

start_link()->
	gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).

%% ====================================================================
%% Server functions
%% ====================================================================

%% --------------------------------------------------------------------
%% Function: init/1
%% Description: Initiates the server
%% Returns: {ok, State}          |
%%          {ok, State, Timeout} |
%%          ignore               |
%%          {stop, Reason}
%% --------------------------------------------------------------------
init([]) ->
	CreateOptions = [named_table, duplicate_bag, public, {write_concurrency, true}, {read_concurrency, true}],
	create_table(CreateOptions, ?TABLE_COUNT),
    {ok, #state{}}.

%% --------------------------------------------------------------------
%% Function: handle_call/3
%% Description: Handling call messages
%% Returns: {reply, Reply, State}          |
%%          {reply, Reply, State, Timeout} |
%%          {noreply, State}               |
%%          {noreply, State, Timeout}      |
%%          {stop, Reason, Reply, State}   | (terminate/2 is called)
%%          {stop, Reason, State}            (terminate/2 is called)
%% --------------------------------------------------------------------
handle_call(_Request, _From, State) ->
    Reply = ok,
    {reply, Reply, State}.

%% --------------------------------------------------------------------
%% Function: handle_cast/2
%% Description: Handling cast messages
%% Returns: {noreply, State}          |
%%          {noreply, State, Timeout} |
%%          {stop, Reason, State}            (terminate/2 is called)
%% --------------------------------------------------------------------
handle_cast(_Msg, State) ->
    {noreply, State}.

%% --------------------------------------------------------------------
%% Function: handle_info/2
%% Description: Handling all non call/cast messages
%% Returns: {noreply, State}          |
%%          {noreply, State, Timeout} |
%%          {stop, Reason, State}            (terminate/2 is called)
%% --------------------------------------------------------------------
handle_info(_Info, State) ->
    {noreply, State}.

%% --------------------------------------------------------------------
%% Function: terminate/2
%% Description: Shutdown the server
%% Returns: any (ignored by gen_server)
%% --------------------------------------------------------------------
terminate(_Reason, _State) ->
    ok.

%% --------------------------------------------------------------------
%% Func: code_change/3
%% Purpose: Convert process state when code is changed
%% Returns: {ok, NewState}
%% --------------------------------------------------------------------
code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

%% --------------------------------------------------------------------
%%% Internal functions
%% --------------------------------------------------------------------
create_table(_CreateOptions, 0)->
	ok;
create_table(CreateOptions, Number)->
	StringNumber = integer_to_list(Number),
	TableName = ?TABLE_PREFIX ++ StringNumber,
	TableAtom = list_to_atom(TableName),
	ets:new(TableAtom, CreateOptions),
	create_table(CreateOptions, Number-1).
